@@ -1,4 +1,5 @@
 # flake8: noqa: E402 (matplotlib.use has to be right after the import)
+import os.path as os_path
 import pathlib
 import pkg_resources
 import signal
@@ -10,6 +11,7 @@ import matplotlib
 matplotlib.use('QT5Agg')
 
 from PyQt5 import uic, QtCore, QtWidgets
+from PyQt5.QtCore import QStandardPaths
 
 import afmformats
 import h5py
@@ -22,7 +24,10 @@ import sklearn
 
 from . import custom_widgets
 from .dlg_tool_convert import ConvertDialog
+from . import preferences
+from . import update
 
+from ..extensions import ExtensionManager
 from .. import registry
 from .._version import version as __version__
 
@@ -56,6 +61,10 @@ class PyJibe(QtWidgets.QMainWindow):
         self.settings = QtCore.QSettings()
         self.settings.setIniCodec("utf-8")
 
+        # update check
+        self._update_thread = None
+        self._update_worker = None
+
         # load ui files
         path_ui = pkg_resources.resource_filename("pyjibe.head", "main.ui")
         uic.loadUi(path_ui, self)
@@ -69,10 +78,7 @@ class PyJibe(QtWidgets.QMainWindow):
         self.action_open_single.triggered.connect(self.on_open_single)
         self.action_open_multiple.triggered.connect(self.on_open_multiple)
         # Edit menu
-        is_dev_mode = bool(int(self.settings.value("developer mode", "0")))
-        self.action_developer_mode.setChecked(is_dev_mode)
-        self.on_developer_mode(is_dev_mode)
-        self.action_developer_mode.triggered.connect(self.on_developer_mode)
+        self.actionPreferences.triggered.connect(self.on_preferences)
         # Tool menu
         self.actionConvert_AFM_data.triggered.connect(self.on_tool_convert)
         # Help menu
@@ -80,6 +86,20 @@ class PyJibe(QtWidgets.QMainWindow):
         self.actionSoftware.triggered.connect(self.on_software)
         self.actionAbout.triggered.connect(self.on_about)
 
+        #: Extensions
+        store_path = os_path.join(
+            QStandardPaths.writableLocation(
+                QStandardPaths.AppLocalDataLocation), "extensions")
+        try:
+            self.extensions = ExtensionManager(store_path)
+        except BaseException:
+            QtWidgets.QMessageBox.warning(
+                self,
+                "Extensions automatically disabled",
+                "Some extensions could not be loaded and were disabled:\n\n"
+                + traceback.format_exc(),
+                )
+
         self.subwindows = []
         self.subwindow_data = []
         self.mdiArea.cascadeSubWindows()
@@ -90,6 +110,11 @@ class PyJibe(QtWidgets.QMainWindow):
             QtWidgets.QApplication.processEvents(
                 QtCore.QEventLoop.AllEvents, 300)
             sys.exit(0)
+
+        # check for updates
+        do_update = int(self.settings.value("check for updates", 1))
+        self.on_action_check_update(do_update)
+
         self.show()
         self.raise_()
         self.activateWindow()
@@ -165,23 +190,49 @@ class PyJibe(QtWidgets.QMainWindow):
                                     about_text)
 
     @QtCore.pyqtSlot(bool)
-    def on_developer_mode(self, checked):
-        current_value = bool(int(self.settings.value("developer mode", "0")))
-        # remember in settings
-        self.settings.setValue("developer mode", str(int(checked)))
-        # set nanite
-        if checked:
-            nanite.read.DEFAULT_MODALITY = None
+    def on_action_check_update(self, b):
+        self.settings.setValue("check for updates", int(b))
+        if b and self._update_thread is None:
+            self._update_thread = QtCore.QThread()
+            self._update_worker = update.UpdateWorker()
+            self._update_worker.moveToThread(self._update_thread)
+            self._update_worker.finished.connect(self._update_thread.quit)
+            self._update_worker.data_ready.connect(
+                self.on_action_check_update_finished)
+            self._update_thread.start()
+
+            version = __version__
+            ghrepo = "AFM-analysis/PyJibe"
+
+            QtCore.QMetaObject.invokeMethod(self._update_worker,
+                                            'processUpdate',
+                                            QtCore.Qt.QueuedConnection,
+                                            QtCore.Q_ARG(str, version),
+                                            QtCore.Q_ARG(str, ghrepo),
+                                            )
+
+    @QtCore.pyqtSlot(dict)
+    def on_action_check_update_finished(self, mdict):
+        # cleanup
+        self._update_thread.quit()
+        self._update_thread.wait()
+        self._update_worker = None
+        self._update_thread = None
+        # display message box
+        ver = mdict["version"]
+        web = mdict["releases url"]
+        dlb = mdict["binary url"]
+        msg = QtWidgets.QMessageBox()
+        msg.setWindowTitle("PyJibe {} available!".format(ver))
+        msg.setTextFormat(QtCore.Qt.RichText)
+        text = "You can install PyJibe {} ".format(ver)
+        if dlb is not None:
+            text += 'from a <a href="{}">direct download</a>. '.format(dlb)
         else:
-            nanite.read.DEFAULT_MODALITY = "force-distance"
-        if current_value != checked:
-            # tell the user that changes are not affecting current analyses
-            QtWidgets.QMessageBox.information(
-                self,
-                "You may need to restart PyJibe",
-                "Changing developer mode may not affect current analysis "
-                + "windows. Please restart PyJibe to make sure it is applied "
-                + "correctly.")
+            text += 'by running `pip install --upgrade pyjibe`. '
+        text += 'Visit the <a href="{}">official release page</a>!'.format(web)
+        msg.setText(text)
+        msg.exec_()
 
     @QtCore.pyqtSlot()
     def on_documentation(self):
@@ -236,6 +287,12 @@ class PyJibe(QtWidgets.QMainWindow):
             self.settings.setValue("paths/load data",
                                    str(pathlib.Path(n[0]).parent))
 
+    @QtCore.pyqtSlot()
+    def on_preferences(self):
+        """Show the DCOR import dialog"""
+        dlg = preferences.Preferences(self)
+        dlg.exec()
+
     @QtCore.pyqtSlot()
     def on_software(self):
         libs = [afmformats,
